@@ -1,5 +1,5 @@
 package DBIx::Perlish;
-# $Id: Perlish.pm,v 1.76 2007/06/22 13:26:20 tobez Exp $
+# $Id: Perlish.pm,v 1.78 2007/07/11 14:57:53 tobez Exp $
 
 use 5.008;
 use warnings;
@@ -10,7 +10,7 @@ use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS $SQL @BIND_VALUES);
 require Exporter;
 use base 'Exporter';
 
-$VERSION = '0.26';
+$VERSION = '0.27';
 @EXPORT = qw(db_fetch db_select db_update db_delete db_insert sql);
 @EXPORT_OK = qw(union intersect except);
 %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
@@ -24,9 +24,9 @@ sub db_update (&) { DBIx::Perlish->update($_[0]) }
 sub db_delete (&) { DBIx::Perlish->delete($_[0]) }
 sub db_insert { DBIx::Perlish->insert(@_) }
 
-sub union (&) {}
-sub intersect (&) {}
-sub except (&) {}
+sub union (&;$) {}
+sub intersect (&;$) {}
+sub except (&;$) {}
 
 my $default_object;
 
@@ -294,7 +294,7 @@ DBIx::Perlish - a perlish interface to SQL databases
 
 =head1 VERSION
 
-This document describes DBIx::Perlish version 0.26
+This document describes DBIx::Perlish version 0.27
 
 
 =head1 SYNOPSIS
